package cn.com.qmth.examcloud.service.core.api;

import io.swagger.annotations.ApiOperation;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import javax.servlet.http.HttpServletResponse;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.PageRequest;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import cn.com.qmth.examcloud.common.util.excel.ExcelError;
import cn.com.qmth.examcloud.service.core.assemble.CourseAssembler;
import cn.com.qmth.examcloud.service.core.dto.CourseDTO;
import cn.com.qmth.examcloud.service.core.entity.Course;
import cn.com.qmth.examcloud.service.core.repo.CourseRepo;
import cn.com.qmth.examcloud.service.core.service.CourseService;
import cn.com.qmth.examcloud.service.core.util.ExportUtils;

/**
 * 课程服务API
 * Created by songyue on 17/1/14.
 */
@RestController
@RequestMapping("${app.api.root}/course")
public class CourseApi {

    @Autowired
    CourseRepo courseRepo;
    @Autowired
    CourseService courseService;
    @Autowired
    CourseAssembler courseAssembler;

    @ApiOperation(value="查询课程分页带查询",notes="分页带查询")
    @GetMapping("/all/{curPage}/{pageSize}")
    public ResponseEntity getAllOrg(@ModelAttribute Course course, @PathVariable Integer curPage,@PathVariable Integer pageSize){
        return new ResponseEntity(courseService.findAlL(course,new PageRequest(curPage - 1,pageSize)), HttpStatus.OK);
    }
    
    @ApiOperation(value="查询课程不分页带查询",notes = "不分页带查询")
    @GetMapping("/all")
    public ResponseEntity getAllExam(@ModelAttribute Course course){
        return new ResponseEntity(courseService.findAlL(course), HttpStatus.OK);
    }
    
    @ApiOperation(value="按ID查询课程",notes="ID查询")
    @GetMapping("/{id}")
    public ResponseEntity getCourseById(@PathVariable Long id){
        return new ResponseEntity(courseRepo.findOne(id),HttpStatus.OK);
    }
    
    @ApiOperation(value="按code查询课程",notes="code查询")
    @GetMapping()
    public ResponseEntity getCourseByCode(@RequestParam Long orgId,@RequestParam String code){
        return new ResponseEntity(courseRepo.findByOrgIdAndCode(orgId, code),HttpStatus.OK);
    }

    @ApiOperation(value="新增课程",notes="新增")
    @PostMapping
    public ResponseEntity addCourse(@RequestBody Course course){
    	course.setCreateTime(new Date());
        return new ResponseEntity(courseRepo.save(course),HttpStatus.CREATED);
    }

    @ApiOperation(value="更新课程",notes="更新")
    @PutMapping
    public ResponseEntity updateCourse(@RequestBody Course course){
    	course.setUpdateTime(new Date());
        return new ResponseEntity(courseRepo.save(course),HttpStatus.OK);
    }

    @ApiOperation(value="按ID删除课程",notes="删除")
    @DeleteMapping("/{id}")
    public ResponseEntity deleteCourse(@PathVariable Long id){
        courseRepo.delete(id);
        return new ResponseEntity(HttpStatus.OK);
    }
    
    @ApiOperation(value="导入课程",notes = "导入")
    @PostMapping("/import")
    public ResponseEntity importCourse(@RequestParam Long orgId,@RequestParam MultipartFile file){
    	try {
    		List<ExcelError> errors = courseService.importCourse(orgId,file.getInputStream());
    		return new ResponseEntity(errors,HttpStatus.OK);
		} catch (IOException e) {
			e.printStackTrace();
			return new ResponseEntity(HttpStatus.INTERNAL_SERVER_ERROR);
		}
    }
    
    @ApiOperation(value="导出课程",notes = "导出")
    @GetMapping("/export")
    public void exportCourse(@ModelAttribute Course orgCriteria,HttpServletResponse response){
    	List<CourseDTO> list = new ArrayList<CourseDTO>();
    	courseService.findAlL(orgCriteria).forEach(c -> {
    		list.add(courseAssembler.toDTO(c));
        });
    	ExportUtils.exportEXCEL("课程列表", CourseDTO.class, list, response);
    }
}
