package cn.com.qmth.examcloud.service.core.api;

import io.swagger.annotations.ApiOperation;

import java.io.IOException;
import java.util.Date;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.PageRequest;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import cn.com.qmth.examcloud.common.util.excel.ExcelError;
import cn.com.qmth.examcloud.service.core.entity.Org;
import cn.com.qmth.examcloud.service.core.repo.OrgRepo;
import cn.com.qmth.examcloud.service.core.service.OrgService;

/**
 * 机构服务API
 * Created by songyue on 17/1/14.
 */
@RestController
@RequestMapping("${app.api.root}/org")
public class OrgApi {

    @Autowired
    OrgRepo orgRepo;

    @Autowired
    OrgService orgService;

    @ApiOperation(value="查询机构分页带查询",notes="分页带查询")
    @GetMapping("/all/{curPage}/{pageSize}")
    public ResponseEntity getAllOrg(@ModelAttribute Org orgCriteria, @PathVariable Integer curPage,@PathVariable Integer pageSize){
        return new ResponseEntity(orgService.findAlL(orgCriteria,new PageRequest(curPage - 1,pageSize)), HttpStatus.OK);
    }
    
    @ApiOperation(value="查询机构不分页带查询",notes = "不分页带查询")
    @GetMapping("/all")
    public ResponseEntity getAllExam(@ModelAttribute Org orgCriteria){
        return new ResponseEntity(orgService.findAlL(orgCriteria), HttpStatus.OK);
    }

    @ApiOperation(value="按ID查询机构",notes="ID查询")
    @GetMapping("/{id}")
    public ResponseEntity<Org> getOrgById(@PathVariable Long id){
        return new ResponseEntity(orgRepo.findOne(id),HttpStatus.OK);
    }

//    @ApiOperation(value="按parentId查询下属机构不带分页",notes="下属机构查询不带分页")
//    @GetMapping("/sub/{parentId}")
//    public ResponseEntity<Org> getOrgByParentId(@PathVariable Long parentId){
//        return new ResponseEntity(orgRepo.findByParentId(parentId),HttpStatus.OK);
//    }
//
//    @ApiOperation(value="按rootId查询下属机构不带分页",notes="下属机构查询不带分页")
//    @GetMapping("/allSub/{rootId}")
//    public ResponseEntity<Org> getOrgByRootId(@PathVariable Long rootId){
//        return new ResponseEntity(orgRepo.findByRootId(rootId),HttpStatus.OK);
//    }

    @ApiOperation(value="新增机构",notes="新增")
    @PostMapping
    public ResponseEntity addSchool(@RequestBody Org org){
        return new ResponseEntity(orgRepo.save(org),HttpStatus.CREATED);
    }

    @ApiOperation(value="更新机构",notes="更新")
    @PutMapping
    public ResponseEntity updateSchool(@RequestBody Org org){
    	org.setUpdateTime(new Date());
        return new ResponseEntity(orgRepo.save(org),HttpStatus.OK);
    }

    @ApiOperation(value="按ID删除机构",notes="删除")
    @DeleteMapping("/{id}")
    public ResponseEntity deleteSchool(@PathVariable Long id){
        orgRepo.delete(id);
        return new ResponseEntity(HttpStatus.OK);
    }
    
    @ApiOperation(value="按父ID导入子机构",notes="导入子机构")
    @PostMapping("/{id}/import")
    public ResponseEntity importLearnCenter(@PathVariable Long id,@RequestParam MultipartFile file){
    	try {
			List<ExcelError> excelErrors = orgService.importLearnCenter(id,file.getInputStream());
			return new ResponseEntity(excelErrors,HttpStatus.OK);
		} catch (IOException e) {
			e.printStackTrace();
			return new ResponseEntity(HttpStatus.INTERNAL_SERVER_ERROR);
		}
    }
    
    @ApiOperation(value="禁用/启用机构",notes="禁用/启用")
    @PutMapping("/{id}")
    public ResponseEntity enableSchool(@PathVariable Long id,@RequestParam boolean enable){
        return new ResponseEntity(orgService.enableSchool(id,enable),HttpStatus.OK);
    }
    
    @ApiOperation(value="按code查询机构(学习中心)",notes="code查询")
    @GetMapping()
    public ResponseEntity getCourseByCode(@RequestParam Long parentId,@RequestParam String code){
        return new ResponseEntity(orgRepo.findByParentIdAndCode(parentId, code),HttpStatus.OK);
    }
}
