package cn.com.qmth.examcloud.service.core.service;

import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;

import cn.com.qmth.examcloud.common.uac.AccessCtrlUtil;
import cn.com.qmth.examcloud.common.uac.entity.AccessUser;
import cn.com.qmth.examcloud.common.util.ErrorMsg;
import cn.com.qmth.examcloud.common.util.RedisUtil;
import cn.com.qmth.examcloud.service.core.dto.UserInfo;
import cn.com.qmth.examcloud.service.core.entity.Org;
import cn.com.qmth.examcloud.service.core.entity.Student;
import cn.com.qmth.examcloud.service.core.entity.User;
import cn.com.qmth.examcloud.service.core.entity.UserRole;
import cn.com.qmth.examcloud.service.core.enums.UserType;
import cn.com.qmth.examcloud.service.core.params.UserParam;
import cn.com.qmth.examcloud.service.core.repo.OrgRepo;
import cn.com.qmth.examcloud.service.core.repo.StudentRepo;
import cn.com.qmth.examcloud.service.core.repo.UserRepo;
import cn.com.qmth.examcloud.service.core.repo.UserRoleRepo;

/**
 * 用户服务类
 * Created by songyue on 17/1/13.
 */
@Service
public class UserService {

    @Autowired
    UserRepo userRepo;

    @Autowired
    StudentRepo studentRepo;

    @Autowired
    OrgRepo orgRepo;

    @Autowired
    UserRoleRepo userRoleRepo;

    /**
     * 初始化密码
     * @param userId
     */
    public void initPassword(long userId){
        User user = userRepo.findOne(userId);
        //根据用户类型采用不同的初始化策略
        if(user.getType() == UserType.NOT_STUDENT){
            //初始化为默认密码
            user.setPassword(UserParam.DEFAULT_PASSWORD);

        }else if(user.getType() == UserType.STUDENT){
            //截取身份证后6位为学生登录密码
            Student student = studentRepo.findByUserId(userId);
            user.setPassword(StringUtils.substring(student.getIdentityNumber(),-6,-1));
        }
        userRepo.save(user);
    }


    /**
     * 一般登录
     * @param loginName
     * @param password
     * @return
     */
    public ResponseEntity login(String loginName,
                                String password){
        User user = userRepo.findByLoginName(loginName);
        return loginProcess(user,password,null);

    }

    /**
     * 二级登录
     * @param orgId
     * @param loginName
     * @param password
     * @return
     */
    public ResponseEntity login(long orgId,
                                String loginName,
                                String password){
        User user = userRepo.findByRootOrgIdAndLoginName(orgId,loginName);
        return loginProcess(user,password,null);
    }

    /**
     * 登录处理
     * @param user
     * @param password
     * @return
     */
    public ResponseEntity loginProcess(User user,
                                       String password,Long studentId){
        if(user == null){
            return new ResponseEntity(new ErrorMsg("该用户不存在"),HttpStatus.NOT_FOUND);
        }else if(!user.getPassword().equals(password)){
            return new ResponseEntity(new ErrorMsg("密码错误"),HttpStatus.EXPECTATION_FAILED);
        }else{
            String token = AccessCtrlUtil.buildToken();
            createAccessUser(token,user,studentId);
            return new ResponseEntity(getUserInfo(user,token,studentId),HttpStatus.OK);
        }
    }

    /**
     * 创建访问用户并加入K/V缓存
     * @param token
     * @param user
     */
    public void createAccessUser(String token,User user,Long studentId){
        AccessUser accessUser = new AccessUser();
        Set<UserRole> userRoleSet = userRoleRepo.findByUserId(user.getId());
        Map<String,Set<String>> rolesMap = userRoleSet.stream()
                .collect(Collectors.groupingBy(UserRole::getAppCode,
                        Collectors.mapping(UserRole::getRoleCode,Collectors.toSet())));
        accessUser.setLoginName(user.getLoginName());
        accessUser.setOrgId(user.getOrgId());
        accessUser.setRootOrgId(user.getRootOrgId());
        accessUser.setUserId(user.getId());
        accessUser.setRoles(rolesMap);
        accessUser.setToken(token);
        accessUser.setStudentId(studentId);
        RedisUtil.setByte(token,accessUser);
    }

    /**
     * 获取用户信息DTO
     * @param user
     * @param token
     * @return
     */
    public UserInfo getUserInfo(User user,String token,Long studentId){
        UserInfo userInfo = new UserInfo();
        Org org = orgRepo.findOne(user.getOrgId());
        Org rootOrg = orgRepo.findOne(user.getRootOrgId());
        userInfo.setUserId(user.getId());
        userInfo.setOrgId(user.getOrgId());
        userInfo.setRootOrgId(user.getRootOrgId());
        userInfo.setName(user.getName());
        userInfo.setAvatar(user.getAvatar());
        userInfo.setLoginName(user.getLoginName());
        userInfo.setOrgName(org.getName());
        userInfo.setRootOrgLogo(rootOrg.getLogo());
        userInfo.setRootOrgName(rootOrg.getName());
        userInfo.setToken(token);
        userInfo.setType(user.getType().toString());
        return userInfo;
    }

    /**
     * 登出
     * @param accessUser
     */
	public void logout(AccessUser accessUser) {
		RedisUtil.deleteByte(accessUser.getToken());
	}

}
